PublicKey

Struct PublicKey 

pub struct PublicKey {
    point: RistrettoPoint,
    compressed: CompressedRistretto,
}
Public key - obtained using PublicKey::from_hex or PrivateKey::public_key.

Fields

point: RistrettoPointcompressed: CompressedRistretto

Implementations

Source

impl PublicKey

pub fn from_hex(hexstr: &str) -> Option<Self>

Turns a 64 digit hex string into a PublicKey.

Returns None when the hex-encoding is invalid or when the hex-encoding does not encode a valid Ristretto point.

pub fn encrypt(&self, plaintext: RistrettoPoint) -> Triple

Encrypts the given plaintext for this public key. If the plaintext is a random point, consider using Self::encrypt_random.

pub fn encrypt_with_random( &self, r: Scalar, plaintext: RistrettoPoint, ) -> Triple

Like Self::encrypt, but you can specify the random scalar used - which you shouldn’t except to make deterministic tests.

pub fn encrypt_random(&self) -> Triple

Effectively encrypts a random plaintext for this public key.

Instead of picking random Ristretto point M and random scalar r and computing (rB, r * pk + M, self) we pick Ristretto points ek and ct randomly and return (ek, ct, sekf). since this is more efficient, and yields the same distribution.